/// <reference types="node" />
import { UrlWithParsedQuery, UrlObject } from 'url';
import { Middleware, Context } from 'koa';
import Server from 'next/dist/next-server/server/next-server';
declare module 'http' {
    interface IncomingMessage {
        context: Context;
    }
    interface ServerResponse {
        context: Context;
        locals: any;
    }
}
declare module 'koa' {
    interface DefaultContext extends NextRequest {
        render: (view: string, data?: any, parsed?: UrlWithParsedQuery) => Promise<any>;
        renderError: (error: Error | null, data?: any, parsed?: UrlWithParsedQuery) => Promise<any>;
        render404: (parsed?: UrlWithParsedQuery) => Promise<any>;
        renderToHTML: (view: string, data?: any) => Promise<any>;
        renderErrorToHTML: (err: any, data?: any) => Promise<any>;
        renderRedirect: (url?: RedirectUrl) => Promise<any>;
        handleNext: (parsed?: UrlWithParsedQuery) => Promise<any>;
    }
    interface NextRequest {
        buildId: string;
        nextApp: NextApp;
    }
    interface BaseRequest extends NextRequest {
    }
    interface BaseResponse {
        _explicitStatus?: number;
    }
}
export interface NextKoaConfig {
    /**
     * how to fetch the json state of the page\
     * `param`: use ?next_fetch=json to fetch\
     * `header`: use X-Requested-With header to fetch\
     * `undefined`: will indicate to use `header` instead
     */
    nextFetch?: 'param' | 'header';
}
export interface PublicConfig {
    /**
     * the next-koa public config\
     * use below method to fetch config under nextjs pages
     * ```
     * import nextKoaConfig from 'next-koa/config'
     * // ... at some where
     * const config = nextKoaConfig()
     * ```
     */
    nextKoaConfig?: NextKoaConfig;
}
export interface ServerConfig {
}
export interface NextConfig {
    useFileSystemPublicRoutes?: boolean;
    publicRuntimeConfig?: PublicConfig;
    serverRuntimeConfig?: ServerConfig;
    assetPrefix?: string;
}
export interface NextKoaStaticOptions {
    extentions?: string[];
    allowOriginExtentions?: string[];
    allowOriginRegex?: RegExp[];
}
export interface NextKoaOptions extends NextKoaStaticOptions {
    name?: string;
    dev?: boolean;
    quiet?: boolean;
    conf?: any;
    dir?: string;
}
export interface NextApp extends Server {
    nextConfig: NextConfig;
    middleware: Middleware<any, Context>;
}
export declare type RedirectUrl = (UrlObject & {
    back?: boolean;
    asPath?: string;
}) | string;
/**
 * a next.js middleware for koa
 * features:
 * 1. support ctx.render / ctx.renderError / ctx.render404 / ctx.handleNext
 * 2. render data can be read through fetchState(ctx)
 * 3. support fetchState inside getInitialProps when rendered on client side
 */
export default function NextKoa(options?: NextKoaOptions): NextApp;
